@@ -4,7 +4,7 @@
 
     Utilities parsing and analyzing Python code.
 
-    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 import inspect
@@ -12,15 +12,14 @@ import itertools
 import re
 import sys
 import tokenize
+from collections import OrderedDict
 from inspect import Signature
-from token import NAME, NEWLINE, INDENT, DEDENT, NUMBER, OP, STRING
+from token import DEDENT, INDENT, NAME, NEWLINE, NUMBER, OP, STRING
 from tokenize import COMMENT, NL
 from typing import Any, Dict, List, Optional, Tuple
 
 from sphinx.pycode.ast import ast  # for py37 or older
 from sphinx.pycode.ast import parse, unparse
-from sphinx.util.inspect import signature_from_ast
-
 
 comment_re = re.compile('^\\s*#: ?(.*)\r?\n?$')
 indent_re = re.compile('^\\s*$')
@@ -229,7 +228,7 @@ class VariableCommentPicker(ast.NodeVisitor):
         self.context = []               # type: List[str]
         self.current_classes = []       # type: List[str]
         self.current_function = None    # type: ast.FunctionDef
-        self.comments = {}              # type: Dict[Tuple[str, str], str]
+        self.comments = OrderedDict()   # type: Dict[Tuple[str, str], str]
         self.annotations = {}           # type: Dict[Tuple[str, str], str]
         self.previous = None            # type: ast.AST
         self.deforders = {}             # type: Dict[str, int]
@@ -262,6 +261,8 @@ class VariableCommentPicker(ast.NodeVisitor):
             self.finals.append(".".join(qualname))
 
     def add_overload_entry(self, func: ast.FunctionDef) -> None:
+        # avoid circular import problem
+        from sphinx.util.inspect import signature_from_ast
         qualname = self.get_qualname_for(func.name)
         if qualname:
             overloads = self.overloads.setdefault(".".join(qualname), [])
