@@ -4,7 +4,7 @@
 
     Sphinx test suite utilities
 
-    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 import functools
@@ -13,7 +13,7 @@ import re
 import sys
 import warnings
 from io import StringIO
-from typing import Any, Dict, Generator, IO, List, Pattern
+from typing import IO, Any, Dict, Generator, List, Pattern
 from xml.etree import ElementTree
 
 from docutils import nodes
@@ -26,7 +26,6 @@ from sphinx.pycode import ModuleAnalyzer
 from sphinx.testing.path import path
 from sphinx.util.osutil import relpath
 
-
 __all__ = [
     'Struct',
     'SphinxTestApp', 'SphinxTestAppWrapperForSkipBuilding',
